<?xml version="1.0"?>
<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
 
  <!-- count in 5 minute increments 
       with 6:00 A.M. as the zero point. Thus this sets
       the start-time to 7:00 PM and the end time to
       9:00 PM. However, these parameters can be 
       adjusted when the stylesheet is invoked. -->
  <xsl:param name="start"  select="156"/>  
  <xsl:param name="finish" select="180"/> 

  <xsl:template match="SCHEDULE">
    <HTML>
      <HEAD>
        <TITLE>
          TV Listings <xsl:value-of select="@DATE"/>
        </TITLE>
      </HEAD>
      <BODY>
        <H1 STYLE="text-align: center">
           TV Listings <xsl:value-of select="@DATE"/>
        </H1>
          
        <TABLE CELLSPACING="0" RULES="all" FRAME="box">
          <COLGROUP>
             <COL WIDTH="20"/>
             <COL WIDTH="10"/>
          </COLGROUP>
          <COLGROUP SPAN="{$finish - $start}" WIDTH="20"/>
          <THEAD STYLE="text-align: center">
            <TR>
              <TD />
              <TD />
              <xsl:call-template name="fillTableHead"/>
            </TR>
          </THEAD>
          <TBODY>
          <xsl:for-each select="STATION">
            <xsl:sort select="@CHANNEL" data-type="number"/>
            <TR>
              <TD STYLE="color: white; background-color: grey;
                          font-weight: bold">
                <xsl:if test="@NETWORK=''">
                  <xsl:value-of select="@CALL_LETTERS"/>            
                </xsl:if>
                <xsl:value-of select="@NETWORK"/>
              </TD>
              <TD STYLE="color: white; background-color: grey;
                          font-weight: bold">
                 <xsl:value-of select="@CHANNEL"/>       
              </TD>  
              
              <xsl:for-each select="SHOW">
                <xsl:variable name="showstart">
                  <xsl:call-template name="getLocalTime">
                    <xsl:with-param name="input" 
                                    select="@START_TIME"/>
                  </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="showlength" select=
                   "number(substring-before(@LENGTH, ' ')) div 5"/>
                <xsl:variable name="realshowlength">
                  <xsl:call-template name="getRealShowLength">
                    <xsl:with-param name="showstart" 
                                    select="$showstart"/>
                    <xsl:with-param name="showlength" 
                                    select="$showlength"/>
                  </xsl:call-template>
                </xsl:variable>
              
                <TD COLSPAN="{$realshowlength}"  valign="top">
                  <B><xsl:value-of select="@NAME"/></B>
                  <xsl:if test="@CLOSED_CAPTIONED='Yes'"> 
                    (CC) 
                  </xsl:if>
                  <xsl:value-of select="@DESCRIPTION"/>
                </TD>  
              </xsl:for-each>
              
           </TR>
          </xsl:for-each>
          </TBODY>
        </TABLE>

        <HR/>
        Copyright 2003
        <A HREF="http://www.elharo.com">
         Elliotte Rusty Harold
        </A>
        <BR/>
        <A HREF="mailto:elharo@metalab.unc.edu">
         elharo@metalab.unc.edu
        </A>

      </BODY>
    </HTML>
  </xsl:template>
  
  <xsl:template name="getLocalTime"> 
     <!-- returns number of five-minute increments 
          since 6:00 A.M. -->
    <xsl:param name="input"/> <!-- in form 19:00-0500 -->
    
    <xsl:variable name="time24" 
                  select="substring-before($input, '-')"/>
    <xsl:variable name="hour" 
                  select="substring-before($time24, ':')"/>
    <xsl:variable name="minutes" 
                  select="substring-after($time24, ':')"/>
    
    <xsl:value-of select="(($hour - 6)*12) + ($minutes div 5)"/>
    
  </xsl:template>

  <xsl:template name="getRealShowLength">
    <xsl:param name="showstart"/> 
    <xsl:param name="showlength"/> 

    <xsl:choose>
      <xsl:when test="$showstart &lt; $start">
         <xsl:value-of 
           select="$showlength - ($start - $showstart)"/>
      </xsl:when>
      <xsl:otherwise>
         <xsl:value-of select="$showlength"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Note use of recursion -->
  <xsl:template name="fillTableHead">
    <xsl:param name="time" select="$start"/>
    <TD COLSPAN="6">
      <xsl:call-template name="formatTime">
        <xsl:with-param name="time" select="$time"/>
      </xsl:call-template>
    </TD>
    <xsl:if test="$time &lt; $finish - 6">
      <xsl:call-template name="fillTableHead">
        <xsl:with-param name="time" select="$time + 6"/>
      </xsl:call-template>
    </xsl:if>     
  </xsl:template>
  
  <xsl:template name="formatTime">
    <xsl:param name="time"/>
    <xsl:variable name="minutes" select="($time * 5) mod 60"/>
    <xsl:variable name="hours" 
                  select="(floor(($time div 12) + 6)) mod 12"/>
    <xsl:value-of select="$hours"/>   
    <xsl:value-of select="':'"/>      
    <xsl:value-of select="format-number($minutes, '00')"/>    
    <xsl:if test="$time &lt; 72 or $time &gt;= 216"> AM</xsl:if>      
    <xsl:if test="$time &gt;= 72 and $time &lt; 216"> PM</xsl:if>     
  </xsl:template>
  
</xsl:stylesheet>